Struct Changed
pub struct Changed<T>(/* private fields */);Expand description
A filter on a component that only retains results the first time after they have been added or mutably dereferenced.
A common use for this filter is avoiding redundant work when values have not changed.
Note that simply mutably dereferencing a component is considered a change (DerefMut).
Bevy does not compare components to their previous values.
To retain all results without filtering but still check whether they were changed after the
system last ran, use Ref<T>.
Note that this includes changes that happened before the first time this Query was run.
Deferred
Note, that entity modifications issued with Commands
(like entity creation or entity component addition or removal) are visible only
after deferred operations are applied, typically after the system that queued them.
Time complexity
Changed is not [ArchetypeFilter], which practically means that
if query (with T component filter) matches million entities,
Changed<T> filter will iterate over all of them even if none of them were changed.
For example, these two systems are roughly equivalent in terms of performance:
fn system1(q: Query<&MyComponent, Changed<Transform>>) {
for item in &q { /* component changed */ }
}
fn system2(q: Query<(&MyComponent, Ref<Transform>)>) {
for item in &q {
if item.1.is_changed() { /* component changed */ }
}
}Examples
fn print_moving_objects_system(query: Query<&Name, Changed<Transform>>) {
for name in &query {
println!("Entity Moved: {:?}", name);
}
}