Struct Entity
pub struct Entity { /* private fields */ }Expand description
Unique identifier for an entity in a World.
Note that this is just an id, not the entity itself.
Further, the entity this id refers to may no longer exist in the World.
For more information about entities, their ids, and how to use them, see the module docs.
Aliasing
Once an entity is despawned, it ceases to exist.
However, its Entity id is still present, and may still be contained in some data.
This becomes problematic because it is possible for a later entity to be spawned at the exact same id!
If this happens, which is rare but very possible, it will be logged.
Aliasing can happen without warning.
Holding onto a Entity id corresponding to an entity well after that entity was despawned can cause un-intuitive behavior for both ordering, and comparing in general.
To prevent these bugs, it is generally best practice to stop holding an Entity or [EntityGeneration] value as soon as you know it has been despawned.
If you must do otherwise, do not assume the Entity id corresponds to the same entity it originally did.
See [EntityGeneration]’s docs for more information about aliasing and why it occurs.
Stability warning
For all intents and purposes, Entity should be treated as an opaque identifier. The internal bit
representation is liable to change from release to release as are the behaviors or performance
characteristics of any of its trait implementations (i.e. Ord, Hash, etc.). This means that changes in
Entity’s representation, though made readable through various functions on the type, are not considered
breaking changes under SemVer.
In particular, directly serializing with Serialize and Deserialize make zero guarantee of long
term wire format compatibility. Changes in behavior will cause serialized Entity values persisted
to long term storage (i.e. disk, databases, etc.) will fail to deserialize upon being updated.
Usage
This data type is returned by iterating a Query that has Entity as part of its query fetch type parameter (learn more).
It can also be obtained by calling EntityCommands::id or EntityWorldMut::id.
fn setup(mut commands: Commands) {
// Calling `spawn` returns `EntityCommands`.
let entity = commands.spawn(SomeComponent).id();
}
fn exclusive_system(world: &mut World) {
// Calling `spawn` returns `EntityWorldMut`.
let entity = world.spawn(SomeComponent).id();
}It can be used to refer to a specific entity to apply EntityCommands, or to call Query::get (or similar methods) to access its components.
fn dispose_expired_food(mut commands: Commands, query: Query<Entity, With<Expired>>) {
for food_entity in &query {
commands.entity(food_entity).despawn();
}
}Implementations
impl Entity
impl Entity
pub const PLACEHOLDER: Entity
pub const PLACEHOLDER: Entity
An entity ID with a placeholder value. This may or may not correspond to an actual entity, and should be overwritten by a new value before being used.
Examples
Initializing a collection (e.g. array or Vec) with a known size:
// Create a new array of size 10 filled with invalid entity ids.
let mut entities: [Entity; 10] = [Entity::PLACEHOLDER; 10];
// ... replace the entities with valid ones.Deriving [Reflect] for a component that has an Entity field:
#[derive(Reflect, Component)]
#[reflect(Component)]
pub struct MyStruct {
pub entity: Entity,
}
impl FromWorld for MyStruct {
fn from_world(_world: &mut World) -> Self {
Self {
entity: Entity::PLACEHOLDER,
}
}
}pub const fn from_index_and_generation(
index: EntityIndex,
generation: EntityGeneration,
) -> Entity
pub const fn from_index_and_generation( index: EntityIndex, generation: EntityGeneration, ) -> Entity
Creates a new instance with the given index and generation.
pub const fn from_index(index: EntityIndex) -> Entity
pub const fn from_index(index: EntityIndex) -> Entity
Creates a new entity ID with the specified index and an unspecified generation.
Note
Spawning a specific entity value is rarely the right choice. Most apps should favor
Commands::spawn. This method should generally
only be used for sharing entities across apps, and only when they have a scheme
worked out to share an index space (which doesn’t happen by default).
In general, one should not try to synchronize the ECS by attempting to ensure that
Entity lines up between instances, but instead insert a secondary identifier as
a component.
pub const fn from_raw_u32(index: u32) -> Option<Entity>
pub const fn from_raw_u32(index: u32) -> Option<Entity>
This is equivalent to from_index except that it takes a u32 instead of an [EntityIndex].
Returns None if the index is u32::MAX.
pub const fn to_bits(self) -> u64
pub const fn to_bits(self) -> u64
Convert to a form convenient for passing outside of rust.
Only useful for identifying entities within the same instance of an application. Do not use for serialization between runs.
No particular structure is guaranteed for the returned bits.
pub const fn from_bits(bits: u64) -> Entity
pub const fn from_bits(bits: u64) -> Entity
Reconstruct an Entity previously destructured with Entity::to_bits.
Only useful when applied to results from to_bits in the same instance of an application.
Panics
This method will likely panic if given u64 values that did not come from Entity::to_bits.
pub const fn try_from_bits(bits: u64) -> Option<Entity>
pub const fn try_from_bits(bits: u64) -> Option<Entity>
Reconstruct an Entity previously destructured with Entity::to_bits.
Only useful when applied to results from to_bits in the same instance of an application.
This method is the fallible counterpart to Entity::from_bits.
pub const fn index(self) -> EntityIndex
pub const fn index(self) -> EntityIndex
Return a transiently unique identifier.
See also [EntityIndex].
No two simultaneously-live entities share the same index, but dead entities’ indices may collide with both live and dead entities. Useful for compactly representing entities within a specific snapshot of the world, such as when serializing.
pub const fn index_u32(self) -> u32
pub const fn index_u32(self) -> u32
Equivalent to self.index().index(). See Self::index for details.
pub const fn generation(self) -> EntityGeneration
pub const fn generation(self) -> EntityGeneration
Returns the generation of this Entity’s index. The generation is incremented each time an entity with a given index is despawned. This serves as a “count” of the number of times a given index has been reused (index, generation) pairs uniquely identify a given Entity.