Struct FilteredResources
pub struct FilteredResources<'w, 's> { /* private fields */ }Expand description
Provides read-only access to a set of Resources defined by the contained [Access].
Use FilteredResourcesMut if you need mutable access to some resources.
To be useful as a SystemParam,
this must be configured using a FilteredResourcesParamBuilder
to build the system using a SystemParamBuilder.
Examples
// Use `FilteredResourcesParamBuilder` to declare access to resources.
let system = (FilteredResourcesParamBuilder::new(|builder| {
builder.add_read::<B>().add_read::<C>();
}),)
.build_state(&mut world)
.build_system(resource_system);
world.init_resource::<A>();
world.init_resource::<C>();
fn resource_system(res: FilteredResources) {
// The resource exists, but we have no access, so we can't read it.
assert!(res.get::<A>().is_err());
// The resource doesn't exist, so we can't read it.
assert!(res.get::<B>().is_err());
// The resource exists and we have access, so we can read it.
let c = res.get::<C>().unwrap();
// The type parameter can be left out if it can be determined from use.
let c: Ref<C> = res.get().unwrap();
}This can be used alongside ordinary Res and ResMut parameters if they do not conflict.
let system = (
FilteredResourcesParamBuilder::new(|builder| {
builder.add_read::<A>();
}),
ParamBuilder,
ParamBuilder,
)
.build_state(&mut world)
.build_system(resource_system);
// Read access to A does not conflict with read access to A or write access to B.
fn resource_system(filtered: FilteredResources, res_a: Res<A>, res_mut_b: ResMut<B>) {
let res_a_2: Ref<A> = filtered.get::<A>().unwrap();
}But it will conflict if it tries to read the same resource that another parameter writes.
ⓘ
let system = (
FilteredResourcesParamBuilder::new(|builder| {
builder.add_read::<A>();
}),
ParamBuilder,
)
.build_state(&mut world)
.build_system(invalid_resource_system);
// Read access to A conflicts with write access to A.
fn invalid_resource_system(filtered: FilteredResources, res_mut_a: ResMut<A>) { }Implementations
impl<'w, 's> FilteredResources<'w, 's>
impl<'w, 's> FilteredResources<'w, 's>
pub fn access(&self) -> &Access
pub fn access(&self) -> &Access
Returns a reference to the underlying [Access].
pub fn has_read<R>(&self) -> boolwhere
R: Resource,
pub fn has_read<R>(&self) -> boolwhere
R: Resource,
Returns true if the FilteredResources has access to the given resource.
Note that Self::get() may still return Err if the resource does not exist.